/*
 *                            Chemtool version 1.4.a7
 *
 *      written by Thomas Volk 
 *      Jan 1998
 *      extensions and GTK-based rewrite by Martin Kroeker 1999-2000
 *      with contributions by Radek Liboska
 *      You can use Chemtool under the terms 
 *      of the GNU General Public License
 *
 *      This software comes with ABSOLUTELY NO WARRANTY
 */

#include "ct.h"
#include "bitmap1.h"
#include "templates.h"
#include "chemcurs.h"
#include <stdio.h>
#include <strings.h>
#include <string.h>
#include <ctype.h>
#include <locale.h>
#ifdef DISABLE_NLS
#define _(Text) Text
#else
#include <libintl.h>
#define _(Text) gettext(Text)
#endif
GtkWidget *filew;
GtkWidget *templates;
GtkWidget *messagew, *message;
GtkWidget *expw;
GtkWidget *mainw;
GtkWidget *window;
GtkWidget *scrolled_window;
GtkWidget *hexbutton, *pent1button, *pent2button, *octbutton, *ltextbutton,
  *ctextbutton, *rtextbutton, *bondbutton, *movebutton, *markbutton,
  *rotatebutton, *hflipbutton, *vflipbutton, *copybutton, *addbutton,
  *tempbutton, *cleanbutton, *rescalebutton,*splinebutton;
GtkWidget *scale;
#if 1
GtkWidget *bracketbutton;
#endif  
GtkWidget *drawing_area;
GtkWidget *bondomenu, *bondmenu, *menuitem;
GtkWidget *msgtext;
GdkCursor *cursor_pencil, *cursor_text, *cursor_bonds, *cursor_markTLC,
  *cursor_markBRC, *cursor_move, *cursor_rescale, *cursor_rotate,
  *cursor_busy;
GdkPixmap *source, *mask;
GdkColor fg = { 0, 0, 0, 0 };	/* Black */
GdkColor bg = { 0, 32767, 32767, 65535 };	/* Blue */
GdkColor wh = { 0, 65535, 65535, 65535 };	/* White */
char filename[100];
static int loadsave = 1;
static int expmode = 0;
static int curbond = 0;

int splinepoint=0;
struct {
int x;
int y;
} spline[4] ;

static gint
configure_event (GtkWidget * mainw, GdkEventConfigure * event)
/* callback function to create an empty white drawing area on startup */
{

  if (picture)
    gdk_pixmap_unref (picture);

  picture = gdk_pixmap_new (mainw->window,
			    mainw->allocation.width + 100,
			    mainw->allocation.height + 100, -1);

  gdk_draw_rectangle (picture,
		      mainw->style->white_gc,
		      TRUE, 0, 0,
		      mainw->allocation.width, mainw->allocation.height);
  Display_Mol ();
  return TRUE;
}

static gint
expose_event (GtkWidget * mainw, GdkEventExpose * event)
/* callback function to redraw parts of the drawing area */
{
  gdk_draw_pixmap (mainw->window,
		   mainw->style->fg_gc[GTK_WIDGET_STATE (mainw)],
		   picture,
		   event->area.x, event->area.y,
		   event->area.x, event->area.y,
		   event->area.width, event->area.height);
  return FALSE;
}

void
yesnodialog (void (*YesFunc) (void))
/* Popup to show a warning message when unsaved changes are about to be
   destroyed by Load or Quit (Modal toplevel dialog) */
{
  GtkWidget *label;
  GtkWidget *button;
  GtkWidget *yesno_window;

  yesno_window = gtk_dialog_new ();
  gtk_signal_connect_object (GTK_OBJECT (yesno_window), "destroy",
			     GTK_SIGNAL_FUNC (gtk_grab_remove),
			     GTK_OBJECT (yesno_window));
  gtk_signal_connect_object (GTK_OBJECT (yesno_window), "destroy",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT (yesno_window));
  gtk_container_border_width (GTK_CONTAINER (yesno_window), 5);

  label =
    gtk_label_new (_
		   ("The current drawing is not saved !\nDo you really wish to continue ?"));

  gtk_misc_set_padding (GTK_MISC (label), 10, 10);

  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (yesno_window)->vbox), label, TRUE,
		      TRUE, 0);
  gtk_widget_show (label);

  button = gtk_button_new_with_label (_("Yes"));

  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_grab_remove),
			     GTK_OBJECT (yesno_window));
  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			     GTK_SIGNAL_FUNC (YesFunc),
			     GTK_OBJECT (yesno_window));
  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT (yesno_window));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (yesno_window)->action_area),
		      button, TRUE, TRUE, 0);
  gtk_widget_show (button);

  button = gtk_button_new_with_label (_("No"));

  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_grab_remove),
			     GTK_OBJECT (yesno_window));
  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT (yesno_window));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (yesno_window)->action_area),
		      button, TRUE, TRUE, 0);
  gtk_widget_show (button);

  gtk_widget_realize (yesno_window);
  gtk_grab_add (yesno_window);
  gtk_window_set_transient_for (GTK_WINDOW (yesno_window),
				GTK_WINDOW (gtk_widget_get_toplevel
					    (yesno_window)));
  gtk_window_set_position (GTK_WINDOW (yesno_window), GTK_WIN_POS_MOUSE);
  gtk_widget_show (yesno_window);
}


static gint
button_press_event (GtkWidget * mainw, GdkEventButton * event)
/* Handle mouse button presses depending on current drawing mode */
{
  int event_x, event_y;
int i,spt;
  if (event->button == 1 && picture != NULL)	/* Button 1 actions: */
    {
      event_x = event->x;
      event_y = event->y;

      switch (drawmode)
	{
	case 1:
	  /* Text mode: Insert a label */
	  Add_atom (event_x, event_y);
#ifdef LIBUNDO
	  undo_snapshot ();
#endif
	  break;
	  ;;
	case 0:
	  /* Line mode: show marker boxes around current position */
	default:
	  Set_vector (event_x, event_y);
	  break;
	  ;;
	case 2:
	  /* Bondtype mode: change bond at cursor to current default */
	  Set_bondtype (event_x, event_y, curbond);
#ifdef LIBUNDO
	  undo_snapshot ();
#endif
	  break;
	  ;;
	case 3:
	case 6:
	case 7:
	  /* Move and rotate modes : initialize operation */
	  Set_pmove (event_x, event_y);
	  break;
	  ;;
	case 4:
	  /* Mark mode : Set corner of rectangle */
	  gdk_window_set_cursor (drawing_area->window, cursor_markBRC);
	  Set_start_rec (event_x, event_y);
	  break;
	  ;;
	case 8:
	  spline[splinepoint].x=round_coord (event_x, size_factor);
	  spline[splinepoint].y=round_coord (event_y, size_factor);
	   splinepoint++;                                                                                                      
	  modify=TRUE;
	  if (splinepoint<4) {
	  for (i=1;i<splinepoint;i++)
	      gdk_draw_line (picture, drawing_area->style->black_gc,
	       (gint) (spline[i-1].x * size_factor), (gint) (spline[i-1].y * size_factor),
	       (gint) (spline[i].x * size_factor), (gint) (spline[i].y * size_factor));
	  CopyPlane();
	  }else{
	spt=0;
	if (curbond==8)spt=1;
	if (curbond==9)spt=2;
	if (curbond==10)spt=-1;
	if (curbond==12)spt=-2;
	add_spline(spline[0].x,spline[0].y,spline[1].x,spline[1].y,
	           spline[2].x,spline[2].y,spline[3].x,spline[3].y,
	           spt,0);
	  splinepoint=0;
	  Display_Mol();
	}
	break;
	;;
      }
    }
  if (event->button == 2 && picture != NULL)
    /* Second (middle) button */
    {
      event_x = event->x;
      event_y = event->y;
      if (drawmode == 2)
	{
	  /* in bondtype mode, reverse direction of current bond */
	  Invert_vector (event_x, event_y);
#ifdef LIBUNDO
	  undo_snapshot ();
#endif
	}
      else if (drawmode == 1)
	Fetch_atom (event_x, event_y);
      else
	{
	  /* in all other modes, set bondtype of current bond to next in list */
	  Add_double (event_x, event_y);
#ifdef LIBUNDO
	  undo_snapshot ();
#endif
	}
    }
  if (event->button == 3 && picture != NULL)
/* Button 3 */
    {
      event_x = event->x;
      event_y = event->y;
      switch (drawmode)
	{
	case 0:
	  /* in line mode, delete bond at cursor */
	  Del_vector (event_x, event_y);
#ifdef LIBUNDO
	  undo_snapshot ();
#endif
	  break;
	case 1:
/* in text mode, deletes label at cursor */
	  Del_atom (event_x, event_y);
#ifdef LIBUNDO
	  undo_snapshot ();
#endif
	case 4:
#ifdef LIBUNDO
	  undo_snapshot ();
#endif
	  Del_rec ();
	  break;
	case 3:
	case 6:
/* in mark, move and rotate modes - does nothing */
#ifdef LIBUNDO
	  if (undo_get_undo_count () > 1)
	    undo_undo ();
	  FreePix ();
	  CreatePix ();
	  Display_Mol ();
#endif
	  break;
	  case 8:
	  /* in spline mode, abort current spline*/
	  splinepoint=0;
	  Display_Mol();
	  break;
	default:
	break;
	}
    }
  return TRUE;
}

static gint
button_release_event (GtkWidget * mainw, GdkEventButton * event)
/* completes drawing actions upon release of the mouse button */
{
  int event_x, event_y;

  if (event->button == 1 && picture != NULL)
/* release of the first (left) button */
    {
      event_x = event->x;
      event_y = event->y;
      if (drawmode == 0)
	{
	  /* in line mode, adds bond from previous to current position */
	  Add_vector (event_x, event_y, curbond);	/* add bond, redisplay */

#ifdef LIBUNDO
	  undo_snapshot ();
#endif
	}
      else
	{
	  if (drawmode == 4)
	    {
	      /* in mark mode, defines final position for opposite corner of rectangle */
	      Mark_rec (event_x, event_y);
	      gdk_window_set_cursor (drawing_area->window, cursor_markTLC);
	      FreePix ();
	      CreatePix ();
	      Display_Mol ();
	    }
	}
    }
  return TRUE;
}


static gint
motion_notify_event (GtkWidget * widget, GdkEventMotion * event)
/* handles mouse movement while a button is down */
{
  int x, y;
  GdkModifierType state;

  if (event->is_hint)
    gdk_window_get_pointer (event->window, &x, &y, &state);
  else
    {
      x = event->x;
      y = event->y;
      state = event->state;
    }

  if (state & GDK_BUTTON1_MASK && picture != NULL)
    {
      /* dragging with button 1 (usually left) down */
      switch (drawmode)
	{
	case 0:
	  /* in line mode, draw bond from last to current cursor position */
	  Put_vector (x, y);
	  break;

	case 3:
	  /* in move mode, shift marked fragment to current cursor position */
	  Put_pmove (x, y);
	  break;
	case 4:
	  /* in mark mode, draw rectangle between initial and current position */
	  Put_rec (x, y);
	  break;
	case 6:
	  /* in rotate mode, rotate marked fragment by an amount proportional to
	     the distance between initial and current cursor position */
	  Put_protate (x, y);
	  break;
	case 7:
	  Put_pscale (x, y);
	  break;
	default:
	  break;
	}
    }
  return TRUE;
}

static gint
key_press_event (GtkWidget * mainw, GdkEventKey * event)
/* Handle key presses depending on current drawing mode */
{
/*  GdkModifierType mods;*/
  char label[5];
  int direction;

  if (drawmode == 0 && picture != NULL)	/* Button 1 actions: */
    {

      switch (event->keyval)
	{
	case 'c':
	case 'o':
	case 'n':
	case 's':
	case 'p':
	case 'h':
	case 'f':
	case 'i':
	case 'r':
	case 'b':
	case 'd':
	  direction = Middle_Text;
	  label[0] = toupper (event->keyval);
	  label[1] = '\0';
	  break;
	case 'l':
	  direction = Left_Text;
	  strcpy (label, "Cl");
	  break;
	case '1':
	  direction = Left_Text;
	  strcpy (label, "CH");
	  break;
	case '2':
	  direction = Left_Text;
	  strcpy (label, "CH_2");
	  break;
	case '3':
	  direction = Left_Text;
	  strcpy (label, "CH_3");
	  break;
	default:
	  return FALSE;
	}

      add_char (hp.tx, hp.ty, label, direction, 0);
      Display_Mol ();

#ifdef LIBUNDO
      undo_snapshot ();
#endif
    }

  return TRUE;
}


void
Zoom (GtkWidget * mainw, gpointer inout)
/* increase or decrease zoom scale (two steps to either side ) */
{
/*  fprintf (stderr, "zoom %d\n", (int) inout);*/
  if ((int) inout != 0)
    {
      if (zoom_factor)
	zoom_factor = --zoom_factor;
    }
  else
    {
      if (zoom_factor < 4)
	zoom_factor++;
    }
  FreePix ();
  CreatePix ();
  Display_Mol ();
}

void
CheckAndClear ()
/* called by Clear button, pops up an "are you sure" dialog if the current
   drawing changed since last save, or calls real Clear function directly */
{
  if (!modify || hp.n + hp.nc + hp.nsp == 0)
    {
      Clear ();
    }
  else
    {
      yesnodialog (Clear);
    }
}
void
Clear ( /*GtkWidget *mainw */ )
/* deletes all atoms and bonds from memory and reinitializes drawing area */
{
  clear_data ();
  modify = 0;
  strcpy (filename, _("unnamed"));
  gtk_window_set_title (GTK_WINDOW (window), "Chemtool 1.4.a7");
  FreePix ();
  CreatePix ();
  Display_Mol ();
}

void
do_fw ()
{
  int error;
  char errtext[255];
  size_t msglen = 255;

  strcpy (formula, "could not write to");
  strcpy (weight, filename);
  strcat (weight, ".rad");
  strcpy (eweight, " ?");
  strcpy (compos, " ?");
  error = export_fw (filename);
  if (error)
    {
      snprintf (errtext, msglen, _("\nHelper process failed - %s %s %s %s!"),
		formula, weight, eweight, compos);
    }
  else
    {
      snprintf (errtext, msglen, _("\n%s    %s  %s  %s"), formula, weight,
		eweight, compos);
    }
  gtk_text_insert (GTK_TEXT (msgtext), NULL, NULL, NULL, errtext,
		   strlen (errtext));

}

void
Change_Text (GtkWidget * mainw, gpointer textdir)
/* callback function for the (left, centered,right) text buttons,
   sets drawmode to text with appropriate justification mode and handles 
   'radio button' status of the toolbar line - this is rather  messy, as 
   we have to find out which button to deactivate */
{
/* FIXME - ugly hack for the focus_in on the textbox, which returns an
   event structure instead of a text direction flag */
  if ((int)textdir< -2 ) {
  	 if ( drawmode != 1) 
            gtk_toggle_button_set_active ((GtkToggleButton *) ltextbutton,
                                                    TRUE); 
            return;
            }
  if (drawmode == 3)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) movebutton, FALSE);
    }
  if (drawmode == 4)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) markbutton, FALSE);
    }
  if (drawmode == 6)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) rotatebutton, FALSE);
    }
  if (drawmode == 7)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) rescalebutton, FALSE);
    }
  if (drawmode == 7)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) splinebutton, FALSE);
    }
  if (drawmode == 2)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) bondbutton, FALSE);
    }
  if (drawmode == 0)
    {
      switch (draw_angle)
	{
	case 1:
	  gtk_toggle_button_set_active ((GtkToggleButton *) hexbutton, FALSE);
	  break;
	  ;;
	case 2:
	  gtk_toggle_button_set_active ((GtkToggleButton *) pent1button,
					FALSE);
	  break;
	  ;;
	case 3:
	  gtk_toggle_button_set_active ((GtkToggleButton *) pent2button,
					FALSE);
	  break;
	  ;;
	case 4:
	  gtk_toggle_button_set_active ((GtkToggleButton *) octbutton, FALSE);
	  ;;
	}
    }
  if (drawmode == 1)
    {
      switch (text_direct)
	{
	case 0:
	  gtk_toggle_button_set_active ((GtkToggleButton *) ltextbutton,
					FALSE);
	  break;
	  ;;
	case -1:
	  gtk_toggle_button_set_active ((GtkToggleButton *) ctextbutton,
					FALSE);
	  break;
	  ;;
	case -2:
	  gtk_toggle_button_set_active ((GtkToggleButton *) rtextbutton,
					FALSE);
	  ;;
	}
    }
  drawmode = 1;
  text_direct = (int) textdir;
                                                    
  gdk_window_set_cursor (drawing_area->window, cursor_text);
}

void
set_bond (GtkWidget * mainw, gpointer bondnum)
{
  curbond = (int) bondnum;
}

void
Bondmode (GtkWidget * mainw)
/* Callback to enter bondtype changing mode - handles 'radio button'
   status of the toolbar after checking which button to deactivate */
{
  if (drawmode == 8)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) splinebutton, FALSE);
    }
  if (drawmode == 7)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) rescalebutton, FALSE);
    }
  if (drawmode == 6)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) rotatebutton, FALSE);
    }
  if (drawmode == 3)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) movebutton, FALSE);
    }
  if (drawmode == 4)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) markbutton, FALSE);
    }
/*
   if (drawmode==2) {
   gtk_toggle_button_set_active((GtkToggleButton*) bondbutton,FALSE);
   }
 */
  if (drawmode == 0)
    {
      switch (draw_angle)
	{
	case 1:
	  gtk_toggle_button_set_active ((GtkToggleButton *) hexbutton, FALSE);
	  break;
	  ;;
	case 2:
	  gtk_toggle_button_set_active ((GtkToggleButton *) pent1button,
					FALSE);
	  break;
	  ;;
	case 3:
	  gtk_toggle_button_set_active ((GtkToggleButton *) pent2button,
					FALSE);
	  break;
	  ;;
	case 4:
	  gtk_toggle_button_set_active ((GtkToggleButton *) octbutton, FALSE);
	  ;;
	}
    }
  if (drawmode == 1)
    {
      switch (text_direct)
	{
	case 0:
	  gtk_toggle_button_set_active ((GtkToggleButton *) ltextbutton,
					FALSE);
	  break;
	  ;;
	case -1:
	  gtk_toggle_button_set_active ((GtkToggleButton *) ctextbutton,
					FALSE);
	  break;
	  ;;
	case -2:
	  gtk_toggle_button_set_active ((GtkToggleButton *) rtextbutton,
					FALSE);
	  ;;
	}
    }
  drawmode = 2;
  gdk_window_set_cursor (drawing_area->window, cursor_bonds);
}

void
Movemode (GtkWidget * mainw)
/* Callback function to enter 'move marked fragment' mode - checks and
   updates radio button status of the toolbar */
{
  if (drawmode == 8)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) splinebutton, FALSE);
    }
  if (drawmode == 7)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) rescalebutton, FALSE);
    }
  if (drawmode == 6)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) rotatebutton, FALSE);
    }
  if (drawmode == 4)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) markbutton, FALSE);
    }
  if (drawmode == 2)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) bondbutton, FALSE);
    }
  if (drawmode == 0)
    {
      switch (draw_angle)
	{
	case 1:
	  gtk_toggle_button_set_active ((GtkToggleButton *) hexbutton, FALSE);
	  break;
	  ;;
	case 2:
	  gtk_toggle_button_set_active ((GtkToggleButton *) pent1button,
					FALSE);
	  break;
	  ;;
	case 3:
	  gtk_toggle_button_set_active ((GtkToggleButton *) pent2button,
					FALSE);
	  break;
	  ;;
	case 4:
	  gtk_toggle_button_set_active ((GtkToggleButton *) octbutton, FALSE);
	  ;;
	}
    }
  if (drawmode == 1)
    {
      switch (text_direct)
	{
	case 0:
	  gtk_toggle_button_set_active ((GtkToggleButton *) ltextbutton,
					FALSE);
	  break;
	  ;;
	case -1:
	  gtk_toggle_button_set_active ((GtkToggleButton *) ctextbutton,
					FALSE);
	  break;
	  ;;
	case -2:
	  gtk_toggle_button_set_active ((GtkToggleButton *) rtextbutton,
					FALSE);
	  ;;
	}
    }
  drawmode = 3;
  gdk_window_set_cursor (drawing_area->window, cursor_move);
  Display_Mol();
}

void
Markmode (GtkWidget * mainw)
/* Callback function to set 'mark fragment' mode, checks and updates
   radio button state of the toolbar */
{
  if (drawmode == 8)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) splinebutton, FALSE);
    }
  if (drawmode == 7)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) rescalebutton, FALSE);
    }
  if (drawmode == 6)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) rotatebutton, FALSE);
    }
  if (drawmode == 3)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) movebutton, FALSE);
    }
  if (drawmode == 2)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) bondbutton, FALSE);
    }
  if (drawmode == 0)
    {
      switch (draw_angle)
	{
	case 1:
	  gtk_toggle_button_set_active ((GtkToggleButton *) hexbutton, FALSE);
	  break;
	  ;;
	case 2:
	  gtk_toggle_button_set_active ((GtkToggleButton *) pent1button,
					FALSE);
	  break;
	  ;;
	case 3:
	  gtk_toggle_button_set_active ((GtkToggleButton *) pent2button,
					FALSE);
	  break;
	  ;;
	case 4:
	  gtk_toggle_button_set_active ((GtkToggleButton *) octbutton, FALSE);
	  ;;
	}
    }
  if (drawmode == 1)
    {
      switch (text_direct)
	{
	case 0:
	  gtk_toggle_button_set_active ((GtkToggleButton *) ltextbutton,
					FALSE);
	  break;
	  ;;
	case -1:
	  gtk_toggle_button_set_active ((GtkToggleButton *) ctextbutton,
					FALSE);
	  break;
	  ;;
	case -2:
	  gtk_toggle_button_set_active ((GtkToggleButton *) rtextbutton,
					FALSE);
	  ;;
	}
    }
  drawmode = 4;
  gdk_window_set_cursor (drawing_area->window, cursor_markTLC);
}

void
Rotatemode (GtkWidget * mainw)
/* Callback function to set 'rotate marked fragment' mode - checks and
   updates radio button state of the toolbar */
{
  if (drawmode == 8)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) splinebutton, FALSE);
    }
  if (drawmode == 7)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) rescalebutton, FALSE);
    }
  if (drawmode == 3)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) movebutton, FALSE);
    }
  if (drawmode == 4)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) markbutton, FALSE);
    }

  if (drawmode == 2)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) bondbutton, FALSE);
    }
  if (drawmode == 0)
    {
      switch (draw_angle)
	{
	case 1:
	  gtk_toggle_button_set_active ((GtkToggleButton *) hexbutton, FALSE);
	  break;
	  ;;
	case 2:
	  gtk_toggle_button_set_active ((GtkToggleButton *) pent1button,
					FALSE);
	  break;
	  ;;
	case 3:
	  gtk_toggle_button_set_active ((GtkToggleButton *) pent2button,
					FALSE);
	  break;
	  ;;
	case 4:
	  gtk_toggle_button_set_active ((GtkToggleButton *) octbutton, FALSE);
	  ;;
	}
    }
  if (drawmode == 1)
    {
      switch (text_direct)
	{
	case 0:
	  gtk_toggle_button_set_active ((GtkToggleButton *) ltextbutton,
					FALSE);
	  break;
	  ;;
	case -1:
	  gtk_toggle_button_set_active ((GtkToggleButton *) ctextbutton,
					FALSE);
	  break;
	  ;;
	case -2:
	  gtk_toggle_button_set_active ((GtkToggleButton *) rtextbutton,
					FALSE);
	  ;;
	}
    }
  drawmode = 6;
  gdk_window_set_cursor (drawing_area->window, cursor_rotate);
}

void
Rescalemode (GtkWidget * mainw)
/* Callback function to set 'rescale marked fragment' mode - checks and
   updates radio button state of the toolbar */
{
  if (drawmode == 8)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) splinebutton, FALSE);
    }
  if (drawmode == 3)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) movebutton, FALSE);
    }
  if (drawmode == 4)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) markbutton, FALSE);
    }

  if (drawmode == 2)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) bondbutton, FALSE);
    }

  if (drawmode == 6)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) rotatebutton, FALSE);
    }

  if (drawmode == 0)
    {
      switch (draw_angle)
	{
	case 1:
	  gtk_toggle_button_set_active ((GtkToggleButton *) hexbutton, FALSE);
	  break;
	  ;;
	case 2:
	  gtk_toggle_button_set_active ((GtkToggleButton *) pent1button,
					FALSE);
	  break;
	  ;;
	case 3:
	  gtk_toggle_button_set_active ((GtkToggleButton *) pent2button,
					FALSE);
	  break;
	  ;;
	case 4:
	  gtk_toggle_button_set_active ((GtkToggleButton *) octbutton, FALSE);
	  ;;
	}
    }
  if (drawmode == 1)
    {
      switch (text_direct)
	{
	case 0:
	  gtk_toggle_button_set_active ((GtkToggleButton *) ltextbutton,
					FALSE);
	  break;
	  ;;
	case -1:
	  gtk_toggle_button_set_active ((GtkToggleButton *) ctextbutton,
					FALSE);
	  break;
	  ;;
	case -2:
	  gtk_toggle_button_set_active ((GtkToggleButton *) rtextbutton,
					FALSE);
	  ;;
	}
    }
  drawmode = 7;
  gdk_window_set_cursor (drawing_area->window, cursor_rescale);
}

void
Splinemode (GtkWidget * mainw)
/* Callback function to set 'rescale marked fragment' mode - checks and
   updates radio button state of the toolbar */
{
  if (drawmode == 3)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) movebutton, FALSE);
    }
  if (drawmode == 4)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) markbutton, FALSE);
    }

  if (drawmode == 2)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) bondbutton, FALSE);
    }

  if (drawmode == 6)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) rotatebutton, FALSE);
    }

  if (drawmode == 0)
    {
      switch (draw_angle)
	{
	case 1:
	  gtk_toggle_button_set_active ((GtkToggleButton *) hexbutton, FALSE);
	  break;
	  ;;
	case 2:
	  gtk_toggle_button_set_active ((GtkToggleButton *) pent1button,
					FALSE);
	  break;
	  ;;
	case 3:
	  gtk_toggle_button_set_active ((GtkToggleButton *) pent2button,
					FALSE);
	  break;
	  ;;
	case 4:
	  gtk_toggle_button_set_active ((GtkToggleButton *) octbutton, FALSE);
	  ;;
	}
    }
  if (drawmode == 1)
    {
      switch (text_direct)
	{
	case 0:
	  gtk_toggle_button_set_active ((GtkToggleButton *) ltextbutton,
					FALSE);
	  break;
	  ;;
	case -1:
	  gtk_toggle_button_set_active ((GtkToggleButton *) ctextbutton,
					FALSE);
	  break;
	  ;;
	case -2:
	  gtk_toggle_button_set_active ((GtkToggleButton *) rtextbutton,
					FALSE);
	  ;;
	}
    }
  drawmode = 8;
  gdk_window_set_cursor (drawing_area->window, cursor_pencil);
}

void
Change_Angle (GtkWidget * mainw, gpointer newangle)
/* Callback function to set 'line drawing' mode and appropriate grid
   definition - checks and updates radio button state of the toolbar */
{
  gboolean activate = FALSE;

  if (drawmode == 3)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) movebutton, FALSE);
    }
  if (drawmode == 4)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) markbutton, FALSE);
    }
  if (drawmode == 6)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) rotatebutton, FALSE);
    }
  if (drawmode == 7)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) rescalebutton, FALSE);
    }
  if (drawmode == 8)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) splinebutton, FALSE);
    }
  if (drawmode == 2)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) bondbutton, FALSE);
    }
  if (drawmode == 1)
    {
      gtk_toggle_button_set_active ((GtkToggleButton *) ltextbutton, FALSE);
      gtk_toggle_button_set_active ((GtkToggleButton *) ctextbutton, FALSE);
      gtk_toggle_button_set_active ((GtkToggleButton *) rtextbutton, FALSE);
    }

  drawmode = 0;
  gdk_window_set_cursor (drawing_area->window, cursor_pencil);

  switch (draw_angle)
    {
    case 1:
      gtk_toggle_button_set_active ((GtkToggleButton *) hexbutton, activate);
      break;
      ;;
    case 2:
      gtk_toggle_button_set_active ((GtkToggleButton *) pent1button,
				    activate);
      break;
      ;;
    case 3:
      gtk_toggle_button_set_active ((GtkToggleButton *) pent2button,
				    activate);
      break;
      ;;
    case 4:
      gtk_toggle_button_set_active ((GtkToggleButton *) octbutton, activate);
      ;;
    }
  draw_angle = (int) newangle;

}

void
CheckAndLoad ()
/* Callback function for 'Load' button - pops up an 'are you sure' dialog
   if unsaved changes exist, else calls real load function directly */
{
  if (!modify || hp.n + hp.nc +hp.nsp == 0)
    {
      Load ();
    }
  else
    {
      yesnodialog (Load);
    }
}

void
Load ()
/* called for loading drawings from file - sets i/o mode to loading and 
   pops up the file selection widget, making it modal */
{
  loadsave = 1;
  gtk_window_set_title (GTK_WINDOW (filew), _("Load from file..."));
  gtk_widget_show (filew);
  gtk_window_set_modal (GTK_WINDOW (filew), TRUE);
}

void
Add ()
/* called for inserting drawings at the currently marked position - sets 
   i/o mode to adding and pops up the (modal) file selection dialog */
{
  if (!mark.flag)
    {
      mark.x = hp.tx;
      mark.y = hp.ty;
      mark.flag = TRUE;
    }
  else
    {
      Unmark_all ();
      mark.x = hp.tx;
      mark.y = hp.ty;
      mark.flag = TRUE;
      mark.w = 200;
      mark.h = 200;
    }

  loadsave = 3;
  gtk_window_set_title (GTK_WINDOW (filew), _("Add from file..."));
  gtk_widget_show (filew);
  gtk_window_set_modal (GTK_WINDOW (filew), TRUE);
}

void
show_or_raise (GtkWidget * thewidget)
{
  if (!GTK_WIDGET_MAPPED (thewidget))
    gtk_widget_show (thewidget);
  else
    gdk_window_raise (thewidget->window);
}



void
Add_template (GtkWidget * mainw, gpointer template)
{
  int i, j, n;
  int xdiff, ydiff, xref, yref;
  int x, y, tx, ty, b;
  int x1,y1,x2,y2;
  xdiff = 0;
  ydiff = 0;
  xref = 0;
  yref = 0;
  i = (int) template / 10;
  j = (int) template - i * 10;
  for (n = 0; n < template_nb[i][j]; n++)
    {
      x = template_x[i][j][n];
      y = template_y[i][j][n];
      tx = template_tx[i][j][n];
      ty = template_ty[i][j][n];
      b = template_b[i][j][n];

      if (!mark.flag)
	{
	  mark.x = hp.tx;
	  mark.y = hp.ty;
	}
      else
	{
	  Unmark_all ();
	}

      if (n == 0)
	{
	  xdiff = mark.x - x;
	  ydiff = mark.y - y;
	  xref = x + xdiff;
	  yref = y + ydiff;
	}
      x = x + xdiff;
      y = y + ydiff;
      tx = tx + xdiff;
      ty = ty + ydiff;
      add_struct (x, y, tx, ty, b, 1, 1);
    }
  for (n = 0; n < template_nl[i][j]; n++)
    {
      x = template_lx[i][j][n] + xdiff;
      y = template_ly[i][j][n] + ydiff;
      add_char (x, y, template_lt[i][j][n], template_lo[i][j][n], 1);
    }
  for (n = 0; n < template_ncrv[i][j]; n++)
  	{
  	x = template_crv[i][j][n][0]  + xdiff;
  	y = template_crv[i][j][n][1] + ydiff;  
  	x1 = template_crv[i][j][n][2] + xdiff;
  	y1 = template_crv[i][j][n][3]  + ydiff;  
  	x2 = template_crv[i][j][n][4] + xdiff;
  	y2 = template_crv[i][j][n][5]  + ydiff;  
  	tx = template_crv[i][j][n][6] + xdiff;
  	ty = template_crv[i][j][n][7] + ydiff;  
  	b  = template_crv[i][j][n][8];
      add_spline(x,y,x1,y1,x2,y2,tx,ty,b,1); 
    }
  mark.flag = TRUE;
  mark.x = xref;
  mark.y = yref;
  mark.w = 200;
  mark.h = 200;
  Movemode (mainw);
  gtk_toggle_button_set_active ((GtkToggleButton *) movebutton, TRUE);
  CreatePix ();
  Display_Mol ();
}

void
Save ()
/* Callback for saving a drawing to file - sets i/o mode to saving and pops
   up the (modal) file selection dialog with the current filename as the default */
{
  if (hp.n + hp.nc + hp.nsp == 0)
    return;
  loadsave = 2;
  if (filename)
    gtk_file_selection_set_filename (GTK_FILE_SELECTION (filew), filename);
  gtk_window_set_title (GTK_WINDOW (filew), _("Save as..."));
  gtk_widget_show (filew);
  gtk_window_set_modal (GTK_WINDOW (filew), TRUE);
}

void
Export ()
/* Callback for exporting a drawing to file - pops
   up the (modal) file selection dialog with the current filename as the default */
{
char expn[255];
char *exten[4]={".fig",".tex",".eps",".xbm"};
int i,n;

  if (hp.n + hp.nc + hp.nsp == 0)
    return;

strcpy(expn,filename);
n=strlen(expn);
for (i=0;i<strlen(expn);i++){
	 if(expn[i]=='.') n=i; 
	 if( expn[i]=='/') n=strlen(expn); 
	}
  	expn[n]='\0';
    strcat(expn,exten[expmode]);
	gtk_file_selection_set_filename(GTK_FILE_SELECTION(expw),expn);
  gtk_widget_show (expw);
  gtk_window_set_modal (GTK_WINDOW (filew), TRUE);
}

void
do_export (GtkWidget *mainw,GtkFileSelection * exp)
/* initiates export of current drawing to the foreign format selected via
   the export popup - pops down export dialog, calls appropriate function
   and displays messagebox with file statistics or error message afterwards */
{
  int error=0;
  char errtext[255];
  char expname[255];
  size_t msglen = 255;
  float expscale;
  
  gtk_widget_hide (expw);
  if (hp.n + hp.nc +hp.nsp == 0)
    return;
    strcpy(expname,gtk_file_selection_get_filename(GTK_FILE_SELECTION(exp)));
    
    expscale= gtk_spin_button_get_value_as_int((GtkSpinButton *)scale)/100.;

  switch (expmode)
    {
    case 0:
      error = export_xfig (expname);
      break;
      ;;
    case 1:
      error = export_latex_pic (expname,expscale);
      break;
      ;;
    case 2:
      error = export_ps_pic (expname,expscale);
      break;
      ;;
    case 3:
      /* xbm output is slow - set 'busy' watch-shaped cursor */
/*      gdk_window_set_cursor (drawing_area->window, cursor_busy);*/
/*      while (g_main_iteration (FALSE));*/
      /* allow gtk to redraw after the popup, 
         which may have obscured parts of our molecule */
      if (mark.flag)
	{
	  xbmflag = 1;
	  FreePix ();
	  CreatePix ();
	  Display_Mol ();
	}
      error = export_bitmap (expname);
      if (mark.flag)
	{
	  xbmflag = 0;
	  FreePix ();
	  CreatePix ();
	  Display_Mol ();
	}
/*      gdk_window_set_cursor (drawing_area->window, previous_cursor);*/
	break;
	}	
      if (error)
	{
	  snprintf (errtext, msglen, _("Writing to\n %s\nfailed !\n"),
		    expname);
	}
      else
	{
	  snprintf (errtext, msglen,
		    _
		    ("Drawing exported as\n %s\n (%d bonds, %d labels)\n"),
		    expname, hp.n, hp.nc);
	}
      gtk_label_set_text (GTK_LABEL (message), errtext);
      gtk_widget_show (messagew);
      gtk_grab_add (messagew);
}

void
exp_mode (GtkWidget *mainw, gpointer mode)
/* callback function to set export format selected in the export dialog */
{
char expn[255];
char *exten[4]={".fig",".tex",".eps",".xbm"};
int i,n;

    strcpy(expn,gtk_file_selection_get_filename(GTK_FILE_SELECTION(expw)));
n=strlen(expn);
for (i=0;i<strlen(expn);i++){
	 if(expn[i]=='.') n=i; 
	 if( expn[i]=='/') n=strlen(expn); 
	}
  	expn[n]='\0';
  expmode = (int) mode;
  if (expmode < 0 || expmode > 3)
    expmode = 0;
    strcat(expn,exten[expmode]);

	gtk_file_selection_set_filename(GTK_FILE_SELECTION(expw),expn);
}

int
CheckAndQuit ()
/* Callback function for the Quit button - pops up an 'are you sure' dialog
   if unsaved changes exist, or calls the real quit function directly */
{
  if (!modify || hp.n + hp.nc + hp.nsp == 0)
    {
      Quit ();
      return (FALSE);
    }
  else
    {
      yesnodialog (Quit);
      return (TRUE);
    }
}

void
Quit ()
/* close down gtk function processing and terminate program */
{
  gtk_main_quit ();
  /* gtk_exit (0); */
}

void
file_ok_sel (GtkWidget * mainw, GtkFileSelection * fs)
/* callback function for the Ok button of the file selection dialog - pops
   down the file dialog, gets the name of the selected file, initiates load
   or save operations as defined by the current i/o mode and displays error
   message dialog if necessary */
{
  int error;
  char errtext[255];
  char oldname[100];

  gtk_widget_hide (filew);
  gtk_grab_remove (filew);
  strcpy (oldname, filename);
  strcpy (filename,
	  gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));
  switch (loadsave)
    {
    case 1:
      error = load_mol (filename);
      switch (error)
	{
	case 0:
	  modify = 0;
	  sprintf (errtext, "Chemtool 1.4a7 (%s)", filename);
	  gtk_window_set_title (GTK_WINDOW (window), errtext);
#ifdef LIBUNDO
	  undo_snapshot ();
#endif
	  break;
	case 1:
	  sprintf (errtext, _("Unable to open %s\n"), filename);
	  strcpy (filename, oldname);
	  gtk_label_set_text (GTK_LABEL (message), errtext);
	  gtk_widget_show (messagew);
	  gtk_grab_add (messagew);
	  break;
	case 2:
	  sprintf (errtext, _("%s\n does not appear to be a Chemtool file\n"),
		   filename);
	  strcpy (filename, oldname);
	  gtk_label_set_text (GTK_LABEL (message), errtext);
	  gtk_widget_show (messagew);
	  gtk_grab_add (messagew);
	  break;
	case 3:
	  sprintf (errtext,
		   _
		   ("%s was created by a newer version.\nSome features may be lost.\n"),
		   filename);
	  gtk_label_set_text (GTK_LABEL (message), errtext);
	  gtk_widget_show (messagew);
	  gtk_grab_add (messagew);
	  sprintf (errtext, "Chemtool 1.4a7 (%s)", filename);
	  gtk_window_set_title (GTK_WINDOW (window), errtext);
#ifdef LIBUNDO
	  undo_snapshot ();
#endif
	  break;
	default:
	  clear_data ();
	  sprintf (errtext, _("Error loading %s \n"), filename);
	  gtk_label_set_text (GTK_LABEL (message), errtext);
	  gtk_widget_show (messagew);
	  gtk_grab_add (messagew);
	}
      CreatePix ();
      Display_Mol ();
      break;
      ;;
    case 2:
      error = save_mol (filename, 0);
      if (error)
	{
	  sprintf (errtext, _("Writing to\n %s\nfailed !\n"), filename);
	}
      else
	{
	  sprintf (errtext,
		   _("Drawing saved in\n %s\n (%d bonds, %d labels)\n"),
		   filename, hp.n, hp.nc);
	  modify = 0;
	}
      gtk_label_set_text (GTK_LABEL (message), errtext);
      gtk_widget_show (messagew);
      gtk_grab_add (messagew);
      sprintf (errtext, "Chemtool 1.4a7 (%s)", filename);
      gtk_window_set_title (GTK_WINDOW (window), errtext);
      break;
      ;;
    case 3:
      add_mol (filename);
      strcpy (filename, oldname);
      Movemode (mainw);
      gtk_toggle_button_set_active ((GtkToggleButton *) movebutton, TRUE);
      CreatePix ();
      Display_Mol ();
      ;;
    default:
      break;
      ;;
    }
}

int
main (int argc, char **argv)
/* Main program - initializes widgets and data structures */
{
  int i, j;
  GtkWidget *aboutw;
  GtkWidget *vbox;
  GtkWidget *hbox;
  GtkWidget *button;
  GtkWidget *expbutton[4];
#ifdef MENU
  GtkWidget *menu_bar;
  GtkWidget *file, *file_menu, *file_items;
  GtkWidget *view, *view_menu, *view_items;
  GtkWidget *tools, *tools_menu, *tools_items;
  GtkWidget *help, *help_menu, *help_items;
#else
GtkWidget *loadbutton, *savebutton, *exportbutton, *zoominbutton; 
GtkWidget *centerbutton, *zoomoutbutton, *clearbutton, *fwbutton, 
*quitbutton, *aboutbutton; 
#endif
  GtkWidget *label;
  GdkPixmap *pixmap;
  GtkWidget *pixmapwid;
  GtkAdjustment *adj_scale;
  GSList *group;
  GtkStyle *style;
  GdkBitmap *mask;
  GtkTooltips *tooltips;
  GtkWidget *templatebook, *page, *tvbox, *pbox;
  GtkWidget *vscroll;

  gtk_init (&argc, &argv);	/*initialize GTK environment */

#ifndef DISABLE_NLS
  bindtextdomain ("chemtool", "/usr/share/locale");
  textdomain ("chemtool");
#endif

#ifdef LIBUNDO
  undo_new ("chemtool");
  undo_set_memory_limit (65536);
#endif
  setup_data ();		/*initialize chemtool structs */
#ifdef LIBUNDO
  undo_snapshot ();
#endif

/* 
 * Create the 'About' pop-up 
 */
  aboutw = gtk_dialog_new ();
  label =
    gtk_label_new (_
		   (" Chemtool Version 1.4a7\nby\nMartin Kroeker,\nRadek Liboska\nand\nThomas Volk\n\nhttp://ruby.chemie.uni-freiburg.de/~martin/chemtool/chemtool.html\nhttp://www.uni-ulm.de/~s_tvolk/chemtool/"));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (aboutw)->vbox), label, TRUE, TRUE,
		      0);
  gtk_widget_show (label);
  button = gtk_button_new_with_label ("Ok");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (aboutw)->action_area), button,
		      TRUE, TRUE, 0);
  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_hide),
			     GTK_OBJECT (aboutw));
  gtk_signal_connect_object (GTK_OBJECT (aboutw), "delete_event",
			     GTK_SIGNAL_FUNC (gtk_widget_hide),
			     GTK_OBJECT (aboutw));
  gtk_widget_show (button);

/****************************************************************/



/*
 *  Build the file selection pop-up  
 */


  filew = gtk_file_selection_new ("File selection");
  gtk_signal_connect (GTK_OBJECT (filew), "delete_event",
		      (GtkSignalFunc) gtk_grab_remove, GTK_OBJECT (filew));
  gtk_signal_connect (GTK_OBJECT (filew), "delete_event",
		      (GtkSignalFunc) gtk_widget_hide, GTK_OBJECT (filew));
  /* Connect the ok_button to file_ok_sel function */
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
		      "clicked", (GtkSignalFunc) file_ok_sel, filew);

  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
					 (filew)->cancel_button),
			     "clicked", (GtkSignalFunc) gtk_grab_remove,
			     GTK_OBJECT (filew));
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
					 (filew)->cancel_button),
			     "clicked", (GtkSignalFunc) gtk_widget_hide,
			     GTK_OBJECT (filew));
/********************************************************************/

/*
 *  Build the export pop-up  
 */


  expw = gtk_file_selection_new ("Export");
  gtk_signal_connect (GTK_OBJECT (expw), "delete_event",
		      (GtkSignalFunc) gtk_grab_remove, GTK_OBJECT (expw));
  gtk_signal_connect (GTK_OBJECT (expw), "delete_event",
		      (GtkSignalFunc) gtk_widget_hide, GTK_OBJECT (expw));
  /* Connect the ok_button to file_ok_sel function */
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (expw)->ok_button),
		      "clicked", (GtkSignalFunc) do_export, expw);

  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
					 (expw)->cancel_button),
			     "clicked", (GtkSignalFunc) gtk_grab_remove,
			     GTK_OBJECT (expw));
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
					 (expw)->cancel_button),
			     "clicked", (GtkSignalFunc) gtk_widget_hide,
			     GTK_OBJECT (expw));
			     hbox=gtk_hbox_new(FALSE,0);
  label=gtk_label_new(_("Latex / EPS scale factor :"));
  gtk_box_pack_start(GTK_BOX(hbox), label, TRUE,FALSE, 0);
  gtk_widget_show(label);
  adj_scale=(GtkAdjustment*) gtk_adjustment_new(100,1,200,1,10,0);
  scale=gtk_spin_button_new(adj_scale,0,0);
                                
  gtk_box_pack_start(GTK_BOX(hbox),scale,FALSE,FALSE,0); 
  gtk_widget_show(scale);
  label=gtk_label_new("%");
  gtk_box_pack_start(GTK_BOX(hbox), label,  FALSE,FALSE, 0);
  gtk_widget_show(label);
  label=gtk_label_new("");
  gtk_box_pack_start(GTK_BOX(hbox), label,  TRUE,FALSE, 0);
  gtk_widget_show(label);
  
  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(GTK_FILE_SELECTION(expw)->main_vbox),hbox,TRUE,TRUE,0);
  hbox=gtk_hbox_new(FALSE,0);                        
  expbutton[0] = gtk_radio_button_new_with_label (NULL, "XFig");
  gtk_box_pack_start (GTK_BOX (hbox), expbutton[0], TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (expbutton[0]), "clicked",
		      GTK_SIGNAL_FUNC (exp_mode), (void *) 0);

  gtk_widget_show (expbutton[0]);
  group = gtk_radio_button_group (GTK_RADIO_BUTTON (expbutton[0]));
  expbutton[1] = gtk_radio_button_new_with_label (group, "LaTeX");
  gtk_box_pack_start (GTK_BOX (hbox), expbutton[1], TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (expbutton[1]), "clicked",
		      GTK_SIGNAL_FUNC (exp_mode), (void *) 1);
  gtk_widget_show (expbutton[1]);
  expbutton[2] =
    gtk_radio_button_new_with_label (gtk_radio_button_group
				     (GTK_RADIO_BUTTON (expbutton[0])),
				     "EPS");
  gtk_box_pack_start (GTK_BOX (hbox), expbutton[2], TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (expbutton[2]), "clicked",
		      GTK_SIGNAL_FUNC (exp_mode), (void *) 2);
  gtk_widget_show (expbutton[2]);
  expbutton[3] =
    gtk_radio_button_new_with_label (gtk_radio_button_group
				     (GTK_RADIO_BUTTON (expbutton[0])),
				     "X Bitmap");
  gtk_signal_connect (GTK_OBJECT (expbutton[3]), "clicked",
		      GTK_SIGNAL_FUNC (exp_mode), (void *) 3);
  gtk_box_pack_start (GTK_BOX (hbox), expbutton[3], TRUE, TRUE, 0);
  gtk_widget_show (expbutton[3]);
  gtk_box_pack_start (GTK_BOX(GTK_FILE_SELECTION(expw)->main_vbox),hbox,TRUE,TRUE,0);
  gtk_widget_show(hbox);
/********************************************************************/

/* 
 * Build the (error) message pop-up  
 */

  messagew = gtk_dialog_new ();
  gtk_signal_connect_object (GTK_OBJECT (messagew), "destroy",
			     (GtkSignalFunc) gtk_grab_remove,
			     GTK_OBJECT (messagew));

  gtk_signal_connect_object (GTK_OBJECT (messagew), "destroy",
			     (GtkSignalFunc) gtk_widget_hide,
			     GTK_OBJECT (messagew));

  message = gtk_label_new (_("Unknown error"));
  gtk_misc_set_padding (GTK_MISC (message), 10, 10);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (messagew)->vbox), message, TRUE,
		      TRUE, 0);

  gtk_widget_show (message);

  button = gtk_button_new_with_label ("OK");
  /* Connect the ok_button */
  gtk_signal_connect_object (GTK_OBJECT (button),
			     "clicked", (GtkSignalFunc) gtk_grab_remove,
			     GTK_OBJECT (messagew));
  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			     (GtkSignalFunc) gtk_widget_hide,
			     GTK_OBJECT (messagew));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (messagew)->action_area), button,
		      TRUE, TRUE, 0);
  gtk_widget_show (button);
/********************************************************************/


/*
 * Build the 'Export' pop-up 
 
  expw = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (expw), "Export");
  gtk_widget_set_name (expw, _("Export drawing"));
  evbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (expw), evbox);
  gtk_widget_show (evbox);

  expbutton[0] = gtk_radio_button_new_with_label (NULL, "XFig");
  gtk_box_pack_start (GTK_BOX (evbox), expbutton[0], TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (expbutton[0]), "clicked",
		      GTK_SIGNAL_FUNC (exp_mode), (void *) 0);

  gtk_widget_show (expbutton[0]);
  group = gtk_radio_button_group (GTK_RADIO_BUTTON (expbutton[0]));
  expbutton[1] = gtk_radio_button_new_with_label (group, "LaTeX");
  gtk_box_pack_start (GTK_BOX (evbox), expbutton[1], TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (expbutton[1]), "clicked",
		      GTK_SIGNAL_FUNC (exp_mode), (void *) 1);
  gtk_widget_show (expbutton[1]);
  expbutton[2] =
    gtk_radio_button_new_with_label (gtk_radio_button_group
				     (GTK_RADIO_BUTTON (expbutton[0])),
				     "EPS");
  gtk_box_pack_start (GTK_BOX (evbox), expbutton[2], TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (expbutton[2]), "clicked",
		      GTK_SIGNAL_FUNC (exp_mode), (void *) 2);
  gtk_widget_show (expbutton[2]);
  expbutton[3] =
    gtk_radio_button_new_with_label (gtk_radio_button_group
				     (GTK_RADIO_BUTTON (expbutton[0])),
				     "X Bitmap");
  gtk_signal_connect (GTK_OBJECT (expbutton[3]), "clicked",
		      GTK_SIGNAL_FUNC (exp_mode), (void *) 3);
  gtk_box_pack_start (GTK_BOX (evbox), expbutton[3], TRUE, TRUE, 0);
  gtk_widget_show (expbutton[3]);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (expbutton[expmode]), TRUE);
  button = gtk_button_new_with_label (_("Export"));
  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			     GTK_SIGNAL_FUNC (do_export), GTK_OBJECT (expw));
  gtk_box_pack_start (GTK_BOX (evbox), button, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (button);
  gtk_widget_show (button);
  button = gtk_button_new_with_label (_("Cancel"));
  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_hide),
			     GTK_OBJECT (expw));
  gtk_box_pack_start (GTK_BOX (evbox), button, FALSE, FALSE, 0);
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  gtk_widget_show (button);
  gtk_widget_realize (expw);
  gtk_signal_connect (GTK_OBJECT (expw), "delete_event",
		      GTK_SIGNAL_FUNC (gtk_widget_hide), GTK_OBJECT (expw));
**************************************************************/

/*************************************************************/
/* Popup to show a selection of templates to insert into the drawing */

  templates = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (templates), "Templates");
  gtk_widget_realize (templates);
  tvbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (templates), tvbox);
  gtk_widget_show (tvbox);
  gtk_signal_connect_object (GTK_OBJECT (templates), "delete_event",
			     (GtkSignalFunc) gtk_widget_hide,
			     GTK_OBJECT (templates));
  templatebook = gtk_notebook_new ();
  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (templatebook), GTK_POS_TOP);
  label = gtk_label_new (_("Carbocycles"));
  gtk_widget_show (label);
  page = gtk_frame_new (_("Carbocycles"));
  gtk_widget_show (page);
  gtk_notebook_append_page (GTK_NOTEBOOK (templatebook), page, label);
  pbox = gtk_table_new (5, 5, TRUE);
  gtk_widget_show (pbox);
  for (i = 0; i < 5; i++)
    {
      for (j = 0; j < 5; j++)
	{
	  button = gtk_button_new ();
	  style = gtk_widget_get_style (templates);
	  pixmap = gdk_pixmap_create_from_xpm_d (templates->window, &mask,
						 &style->bg[GTK_STATE_NORMAL],
						 (gchar **)
						 template_xpm[i][j]);
	  pixmapwid = gtk_pixmap_new (pixmap, mask);
	  gtk_widget_show (pixmapwid);
	  gdk_pixmap_unref (pixmap);
	  gtk_container_add (GTK_CONTAINER (button), pixmapwid);
	  gtk_widget_show (button);
	  gtk_table_attach_defaults (GTK_TABLE (pbox), button, j, j + 1, i,
				     i + 1);
	  gtk_signal_connect (GTK_OBJECT (button), "clicked",
			      GTK_SIGNAL_FUNC (Add_template),
			      (void *) (10 * i + j));
	}
    }
  gtk_container_add (GTK_CONTAINER (page), pbox);
  label = gtk_label_new (_("Sugars"));
  gtk_widget_show (label);
  page = gtk_frame_new (_("Sugars"));
  gtk_widget_show (page);
  gtk_notebook_append_page (GTK_NOTEBOOK (templatebook), page, label);
  pbox = gtk_table_new (5, 5, TRUE);
  gtk_widget_show (pbox);
  for (i = 0; i < 5; i++)
    {
      for (j = 0; j < 5; j++)
	{
	  button = gtk_button_new ();
	  style = gtk_widget_get_style (templates);
	  pixmap = gdk_pixmap_create_from_xpm_d (templates->window, &mask,
						 &style->bg[GTK_STATE_NORMAL],
						 (gchar **) template_xpm[i +
									 5]
						 [j]);
	  pixmapwid = gtk_pixmap_new (pixmap, mask);
	  gtk_widget_show (pixmapwid);
	  gdk_pixmap_unref (pixmap);
	  gtk_container_add (GTK_CONTAINER (button), pixmapwid);
	  gtk_widget_show (button);
	  gtk_table_attach_defaults (GTK_TABLE (pbox), button, j, j + 1, i,
				     i + 1);
	  gtk_signal_connect (GTK_OBJECT (button), "clicked",
			      GTK_SIGNAL_FUNC (Add_template),
			      (void *) (10 * (i + 5) + j));
	}
    }
  gtk_container_add (GTK_CONTAINER (page), pbox);

  label = gtk_label_new (_("Heterocycles"));
  gtk_widget_show (label);
  page = gtk_frame_new (_("Heterocycles"));
  gtk_widget_show (page);
  gtk_notebook_append_page (GTK_NOTEBOOK (templatebook), page, label);
  pbox = gtk_table_new (5, 5, TRUE);
  gtk_widget_show (pbox);
  for (i = 0; i < 5; i++)
    {
      for (j = 0; j < 5; j++)
	{
	  button = gtk_button_new ();
	  style = gtk_widget_get_style (templates);
	  pixmap = gdk_pixmap_create_from_xpm_d (templates->window, &mask,
						 &style->bg[GTK_STATE_NORMAL],
						 (gchar **) template_xpm[i +
									 10]
						 [j]);
	  pixmapwid = gtk_pixmap_new (pixmap, mask);
	  gtk_widget_show (pixmapwid);
	  gdk_pixmap_unref (pixmap);
	  gtk_container_add (GTK_CONTAINER (button), pixmapwid);
	  gtk_widget_show (button);
	  gtk_table_attach_defaults (GTK_TABLE (pbox), button, j, j + 1, i,
				     i + 1);
	  gtk_signal_connect (GTK_OBJECT (button), "clicked",
			      GTK_SIGNAL_FUNC (Add_template),
			      (void *) (10 * (i + 10) + j));
	}
    }
  gtk_container_add (GTK_CONTAINER (page), pbox);

  label = gtk_label_new (_("Orbitals"));
  gtk_widget_show (label);
  page = gtk_frame_new (_("Orbitals"));
  gtk_widget_show (page);
  gtk_notebook_append_page (GTK_NOTEBOOK (templatebook), page, label);
  pbox = gtk_table_new (5, 5, TRUE);
  gtk_widget_show (pbox);
  for (i = 0; i < 5; i++)
    {
      for (j = 0; j < 5; j++)
	{
	  button = gtk_button_new ();
	  style = gtk_widget_get_style (templates);
	  pixmap = gdk_pixmap_create_from_xpm_d (templates->window, &mask,
						 &style->bg[GTK_STATE_NORMAL],
						 (gchar **) template_xpm[i +
									 15]
						 [j]);
	  pixmapwid = gtk_pixmap_new (pixmap, mask);
	  gtk_widget_show (pixmapwid);
	  gdk_pixmap_unref (pixmap);
	  gtk_container_add (GTK_CONTAINER (button), pixmapwid);
	  gtk_widget_show (button);
	  gtk_table_attach_defaults (GTK_TABLE (pbox), button, j, j + 1, i,
				     i + 1);
	  gtk_signal_connect (GTK_OBJECT (button), "clicked",
			      GTK_SIGNAL_FUNC (Add_template),
			      (void *) (10 * (i + 15) + j));
	}
    }
  gtk_container_add (GTK_CONTAINER (page), pbox);

  gtk_widget_show (templatebook);
  gtk_box_pack_start (GTK_BOX (tvbox), templatebook, TRUE, TRUE, 0);
  button = gtk_button_new_with_label (_("Close"));
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (tvbox), button, TRUE, TRUE, 0);
  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_hide),
			     GTK_OBJECT (templates));

/**************************************************************/


/*
 * Create the main window 
 */

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_name (window, "Chemtool");
  gtk_window_set_title (GTK_WINDOW (window), "Chemtool 1.4a7");

/* all pop-ups should appear topmost and at the current cursor position,
   that is, near their respective toolbar buttons */
  gtk_window_set_transient_for (GTK_WINDOW (filew), GTK_WINDOW (window));
  gtk_window_set_position (GTK_WINDOW (filew), GTK_WIN_POS_MOUSE);
  gtk_window_set_transient_for (GTK_WINDOW (expw), GTK_WINDOW (window));
  gtk_window_set_position (GTK_WINDOW (expw), GTK_WIN_POS_MOUSE);
  gtk_window_set_transient_for (GTK_WINDOW (messagew), GTK_WINDOW (window));
  gtk_window_set_position (GTK_WINDOW (messagew), GTK_WIN_POS_MOUSE);

  tooltips = gtk_tooltips_new ();	/* initialize tool-tip message boxes */

/* initialize vertical stacking of button rows and drawing area */
  vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (window), vbox);
  gtk_widget_show (vbox);

  gtk_signal_connect (GTK_OBJECT (window), "delete_event",
		      GTK_SIGNAL_FUNC (CheckAndQuit), NULL);

#ifdef MENU
/* build the menubar */
  
  /* the menuitems of the file-menu */
  file = gtk_menu_new ();
  file_items = gtk_menu_item_new_with_label ("New");
  gtk_menu_append (GTK_MENU (file), file_items);
  gtk_signal_connect_object (GTK_OBJECT (file_items), "activate",
			     GTK_SIGNAL_FUNC (CheckAndClear), NULL);
  gtk_widget_show (file_items);
  file_items = gtk_menu_item_new_with_label ("Open");
  gtk_menu_append (GTK_MENU (file), file_items);
  gtk_signal_connect_object (GTK_OBJECT (file_items), "activate",
			     GTK_SIGNAL_FUNC (CheckAndLoad), NULL);
  gtk_widget_show (file_items);
  file_items = gtk_menu_item_new_with_label ("Add");
  gtk_menu_append (GTK_MENU (file), file_items);
  gtk_signal_connect_object (GTK_OBJECT (file_items), "activate",
			     GTK_SIGNAL_FUNC (Add), NULL);
  gtk_widget_show (file_items);
  file_items = gtk_menu_item_new_with_label ("Export...");
  gtk_menu_append (GTK_MENU (file), file_items);
  gtk_signal_connect_object (GTK_OBJECT (file_items), "activate",
			     GTK_SIGNAL_FUNC (Export), NULL);
  gtk_widget_show (file_items);
  file_items = gtk_menu_item_new_with_label ("Save");
  gtk_menu_append (GTK_MENU (file), file_items);
  gtk_signal_connect_object (GTK_OBJECT (file_items), "activate",
			     GTK_SIGNAL_FUNC (Save), NULL);
  gtk_widget_show (file_items);
  file_items = gtk_menu_item_new_with_label ("Quit");
  gtk_menu_append (GTK_MENU (file), file_items);
  gtk_signal_connect_object (GTK_OBJECT (file_items), "activate",
			     GTK_SIGNAL_FUNC (CheckAndQuit), NULL);
  gtk_widget_show (file_items);

  file_menu = gtk_menu_item_new_with_label ("File");
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (file_menu), file);
  gtk_widget_show (file_menu);

  /* the menuitems of the view-menu */
  view = gtk_menu_new ();
  view_items = gtk_menu_item_new_with_label ("Zoom in");
  gtk_menu_append (GTK_MENU (view), view_items);
  gtk_signal_connect (GTK_OBJECT (view_items), "activate",
		      GTK_SIGNAL_FUNC (Zoom), 
		      (void *) 0);
  gtk_widget_show (view_items);
  view_items = gtk_menu_item_new_with_label ("Zoom out");
  gtk_menu_append (GTK_MENU (view), view_items);
  gtk_signal_connect (GTK_OBJECT (view_items), "activate",
		      GTK_SIGNAL_FUNC (Zoom), 
		      (void *) 1);
  gtk_widget_show (view_items);
  view_items = gtk_menu_item_new_with_label ("Center");
  gtk_menu_append (GTK_MENU (view), view_items);
  gtk_signal_connect (GTK_OBJECT (view_items), "activate",
		      GTK_SIGNAL_FUNC (Center), 
		      GTK_OBJECT (window));
  gtk_widget_show (view_items);

  view_menu = gtk_menu_item_new_with_label ("View");
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (view_menu), view);
  gtk_widget_show (view_menu);
  
  /* the menuitems of the tools-menu */
  tools = gtk_menu_new ();
  tools_items = gtk_menu_item_new_with_label ("Templates...");
  gtk_menu_append (GTK_MENU (tools), tools_items);
  gtk_signal_connect_object (GTK_OBJECT (tools_items), "activate",
			     GTK_SIGNAL_FUNC (show_or_raise), 
			     GTK_OBJECT (templates));
  gtk_widget_show (tools_items);
  tools_items = gtk_menu_item_new_with_label ("Calculate Formula Weight");
  gtk_menu_append (GTK_MENU (tools), tools_items);
  gtk_signal_connect (GTK_OBJECT (tools_items), "activate",
		      GTK_SIGNAL_FUNC (do_fw), 
		      NULL);
  gtk_widget_show (tools_items);

  tools_menu = gtk_menu_item_new_with_label ("Tools");
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (tools_menu), tools);
  gtk_widget_show (tools_menu);


  /* the menuitems of the help-menu */
  help = gtk_menu_new ();
  help_items = gtk_menu_item_new_with_label ("About");
  gtk_menu_append (GTK_MENU (help), help_items);
  gtk_signal_connect_object (GTK_OBJECT (help_items), "activate",
			     GTK_SIGNAL_FUNC (gtk_widget_show), 
			     GTK_OBJECT (aboutw));
  gtk_widget_show (help_items);

  help_menu = gtk_menu_item_new_with_label ("Help");
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (help_menu), help);
  gtk_widget_show (help_menu);

  /* build the menubar */
  menu_bar = gtk_menu_bar_new ();
  gtk_box_pack_start (GTK_BOX (vbox), menu_bar, FALSE, FALSE, 2);
  gtk_widget_show (menu_bar);
  gtk_menu_bar_append (GTK_MENU_BAR (menu_bar), file_menu);
  gtk_menu_bar_append (GTK_MENU_BAR (menu_bar), view_menu);
  gtk_menu_bar_append (GTK_MENU_BAR (menu_bar), tools_menu);
  gtk_menu_item_right_justify(GTK_MENU_ITEM(help_menu));
  gtk_menu_bar_append (GTK_MENU_BAR (menu_bar), help_menu);

#else
  
/* initialize horizontal packing of the first row of buttons */
/* define the first row of buttons, with label,callback and tooltip text */
/* Load | Add | Save | Export | ZoomIn | Center | ZoomOut | Clear | Quit | About */
  hbox = gtk_hbox_new (TRUE, 0);
  loadbutton = gtk_button_new_with_label (_("Load"));
  gtk_box_pack_start (GTK_BOX (hbox), loadbutton, FALSE, TRUE, 0);
  gtk_signal_connect_object (GTK_OBJECT (loadbutton), "clicked",
			     GTK_SIGNAL_FUNC (CheckAndLoad), NULL);
  gtk_tooltips_set_tip (tooltips, loadbutton, _("Load a chemtool sketch"),
			NULL);
  gtk_widget_show (loadbutton);

  addbutton = gtk_button_new_with_label (_("Add"));
  gtk_box_pack_start (GTK_BOX (hbox), addbutton, TRUE, TRUE, 0);
  gtk_signal_connect_object (GTK_OBJECT (addbutton), "clicked",
			     GTK_SIGNAL_FUNC (Add), NULL);
  gtk_tooltips_set_tip (tooltips, addbutton, _("Add fragment from file"),
			NULL);
  gtk_widget_show (addbutton);
  tempbutton = gtk_button_new_with_label (_("Templates"));
  gtk_box_pack_start (GTK_BOX (hbox), tempbutton, TRUE, TRUE, 0);
  gtk_signal_connect_object (GTK_OBJECT (tempbutton), "clicked",
			     GTK_SIGNAL_FUNC (show_or_raise),
			     GTK_OBJECT (templates));
  gtk_tooltips_set_tip (tooltips, tempbutton, _("Add template structure"),
			NULL);
  gtk_widget_show (tempbutton);
  savebutton = gtk_button_new_with_label (_("Save"));
  gtk_box_pack_start (GTK_BOX (hbox), savebutton, TRUE, TRUE, 0);
  gtk_signal_connect_object (GTK_OBJECT (savebutton), "clicked",
			     GTK_SIGNAL_FUNC (Save), NULL);
  gtk_tooltips_set_tip (tooltips, savebutton, _("Save sketch to file"), NULL);
  gtk_widget_show (savebutton);

  exportbutton = gtk_button_new_with_label (_("Export"));
  gtk_box_pack_start (GTK_BOX (hbox), exportbutton, TRUE, TRUE, 0);
  gtk_signal_connect_object (GTK_OBJECT (exportbutton), "clicked",
			     GTK_SIGNAL_FUNC (Export),
			     NULL);
  gtk_signal_connect (GTK_OBJECT (exportbutton), "clicked",
			     GTK_SIGNAL_FUNC (gtk_fileselection_set_filename),
			     GTK_OBJECT (expw),filename);
  gtk_tooltips_set_tip (tooltips, exportbutton,
			_("Create EPS, XFig or XBM file"), NULL);
  gtk_widget_show (exportbutton);

  zoominbutton = gtk_button_new_with_label (_("Zoom In"));
  gtk_box_pack_start (GTK_BOX (hbox), zoominbutton, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (zoominbutton), "clicked",
		      GTK_SIGNAL_FUNC (Zoom), (void *) 0);
  gtk_tooltips_set_tip (tooltips, zoominbutton, _("Increase zoom scale"),
			NULL);
  gtk_widget_show (zoominbutton);

  centerbutton = gtk_button_new_with_label (_("Center"));
  gtk_box_pack_start (GTK_BOX (hbox), centerbutton, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (centerbutton), "clicked",
		      GTK_SIGNAL_FUNC (Center), GTK_OBJECT (window));
  gtk_tooltips_set_tip (tooltips, centerbutton,
			_("Center molecule in drawing area"), NULL);
  gtk_widget_show (centerbutton);

  zoomoutbutton = gtk_button_new_with_label (_("Zoom Out"));
  gtk_box_pack_start (GTK_BOX (hbox), zoomoutbutton, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (zoomoutbutton), "clicked",
		      GTK_SIGNAL_FUNC (Zoom), (void *) 1);
  gtk_tooltips_set_tip (tooltips, zoomoutbutton, _("Decrease zoom scale"),
			NULL);
  gtk_widget_show (zoomoutbutton);

  clearbutton = gtk_button_new_with_label (_("Clear"));
  gtk_box_pack_start (GTK_BOX (hbox), clearbutton, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (clearbutton), "clicked",
		      GTK_SIGNAL_FUNC (CheckAndClear), NULL);
  gtk_tooltips_set_tip (tooltips, clearbutton, _("Remove molecule"), NULL);
  gtk_widget_show (clearbutton);

  fwbutton = gtk_button_new_with_label (_("FW"));
  gtk_box_pack_start (GTK_BOX (hbox), fwbutton, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (fwbutton), "clicked",
		      GTK_SIGNAL_FUNC (do_fw), NULL);
  gtk_tooltips_set_tip (tooltips, fwbutton, _("Calculate Formula Mass"),
			NULL);
  gtk_widget_show (fwbutton);

  quitbutton = gtk_button_new_with_label (_("Quit"));
  gtk_box_pack_start (GTK_BOX (hbox), quitbutton, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (quitbutton), "clicked",
		      GTK_SIGNAL_FUNC (CheckAndQuit), NULL);
  gtk_tooltips_set_tip (tooltips, quitbutton, _("Exit Chemtool"), NULL);
  gtk_widget_show (quitbutton);

  aboutbutton = gtk_button_new_with_label (_("About"));
  gtk_box_pack_end (GTK_BOX (hbox), aboutbutton, FALSE, TRUE, 0);
  gtk_signal_connect_object (GTK_OBJECT (aboutbutton), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_show),
			     GTK_OBJECT (aboutw));
  gtk_tooltips_set_tip (tooltips, aboutbutton, _("About Chemtool"), NULL);
  gtk_widget_show (aboutbutton);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
  gtk_widget_show (hbox); 

#endif /* menubar or button row */  

/* end top row */
/* initialize horizontal packing of second row of buttons */
  hbox = gtk_hbox_new (FALSE, 5);

  gtk_widget_realize (window);
  /* complete setup of base window, but do not show it yet - we just
     need a pointer to it and its color settings for the button pixmaps */

/* Second row has pixmap icons for line drawing on hexagonal, pentagonal
   (two orientations) and octagonal grid; left-justfied,centered and
   right-justified text; bondtype, move, mark and rotate mode; 
   vertical and horizontal flipping; copying; and a text entry box */

  style = gtk_widget_get_style (window);

  pixmap = gdk_pixmap_create_from_xpm_d (window->window, &mask,
					 &style->bg[GTK_STATE_NORMAL],
					 (gchar **) xpm_hex);
  pixmapwid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmapwid);
  gdk_pixmap_unref (pixmap);

  hexbutton = gtk_toggle_button_new ();
  gtk_container_add (GTK_CONTAINER (hexbutton), pixmapwid);
  gtk_box_pack_start (GTK_BOX (hbox), hexbutton, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (hexbutton), "clicked",
		      GTK_SIGNAL_FUNC (Change_Angle), (void *) 1);
  gtk_tooltips_set_tip (tooltips, hexbutton,
			_("Draw at 0/30/60/90 degree angles"), NULL);
  gtk_widget_show (hexbutton);

  pent1button = gtk_toggle_button_new ();
  pixmap = gdk_pixmap_create_from_xpm_d (window->window, &mask,
					 &style->bg[GTK_STATE_NORMAL],
					 (gchar **) xpm_pent1);
  pixmapwid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmapwid);
  gdk_pixmap_unref (pixmap);
  gtk_container_add (GTK_CONTAINER (pent1button), pixmapwid);
  gtk_box_pack_start (GTK_BOX (hbox), pent1button, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (pent1button), "clicked",
		      GTK_SIGNAL_FUNC (Change_Angle), (void *) 2);
  gtk_tooltips_set_tip (tooltips, pent1button,
			_("Draw at 0/36/72/... degree angles"), NULL);
  gtk_widget_show (pent1button);

  pent2button = gtk_toggle_button_new ();
  pixmap = gdk_pixmap_create_from_xpm_d (window->window, &mask,
					 &style->bg[GTK_STATE_NORMAL],
					 (gchar **) xpm_pent2);
  pixmapwid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmapwid);
  gdk_pixmap_unref (pixmap);

  gtk_container_add (GTK_CONTAINER (pent2button), pixmapwid);
  gtk_box_pack_start (GTK_BOX (hbox), pent2button, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (pent2button), "clicked",
		      GTK_SIGNAL_FUNC (Change_Angle), (void *) 3);
  gtk_tooltips_set_tip (tooltips, pent2button,
			_("Draw at 18/54/90/... degree angles"), NULL);
  gtk_widget_show (pent2button);

  octbutton = gtk_toggle_button_new ();
  pixmap = gdk_pixmap_create_from_xpm_d (window->window, &mask,
					 &style->bg[GTK_STATE_NORMAL],
					 (gchar **) xpm_octa);
  pixmapwid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmapwid);
  gdk_pixmap_unref (pixmap);

  gtk_container_add (GTK_CONTAINER (octbutton), pixmapwid);
  gtk_box_pack_start (GTK_BOX (hbox), octbutton, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (octbutton), "clicked",
		      GTK_SIGNAL_FUNC (Change_Angle), (void *) 4);
  gtk_tooltips_set_tip (tooltips, octbutton,
			_("Draw at 0/45/90... degree angles"), NULL);
  gtk_widget_show (octbutton);

  splinebutton = gtk_toggle_button_new ();
  pixmap = gdk_pixmap_create_from_xpm_d (window->window, &mask,
					 &style->bg[GTK_STATE_NORMAL],
					 (gchar **) xpm_spline);
  pixmapwid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmapwid);
  gdk_pixmap_unref (pixmap);

  gtk_container_add (GTK_CONTAINER (splinebutton), pixmapwid);
  gtk_box_pack_start (GTK_BOX (hbox), splinebutton, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (splinebutton), "clicked",
		      GTK_SIGNAL_FUNC (Splinemode), NULL);
  gtk_tooltips_set_tip (tooltips, splinebutton,
			_("Draw curves based on 4 control points"), NULL);
  gtk_widget_show (splinebutton);

  ltextbutton = gtk_toggle_button_new ();
  pixmap = gdk_pixmap_create_from_xpm_d (window->window, &mask,
					 &style->bg[GTK_STATE_NORMAL],
					 (gchar **) xpm_ltext);
  pixmapwid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmapwid);
  gdk_pixmap_unref (pixmap);

  gtk_container_add (GTK_CONTAINER (ltextbutton), pixmapwid);
  gtk_box_pack_start (GTK_BOX (hbox), ltextbutton, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (ltextbutton), "clicked",
		      GTK_SIGNAL_FUNC (Change_Text), (void *) 0);
  gtk_tooltips_set_tip (tooltips, ltextbutton, _("Draw left-justified text"),
			NULL);
  gtk_widget_show (ltextbutton);

  ctextbutton = gtk_toggle_button_new ();
  pixmap = gdk_pixmap_create_from_xpm_d (window->window, &mask,
					 &style->bg[GTK_STATE_NORMAL],
					 (gchar **) xpm_mtext);
  pixmapwid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmapwid);
  gdk_pixmap_unref (pixmap);

  gtk_container_add (GTK_CONTAINER (ctextbutton), pixmapwid);
  gtk_box_pack_start (GTK_BOX (hbox), ctextbutton, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (ctextbutton), "clicked",
		      GTK_SIGNAL_FUNC (Change_Text), (void *) -1);
  gtk_tooltips_set_tip (tooltips, ctextbutton, _("Draw centered text"), NULL);
  gtk_widget_show (ctextbutton);


  rtextbutton = gtk_toggle_button_new ();
  pixmap = gdk_pixmap_create_from_xpm_d (window->window, &mask,
					 &style->bg[GTK_STATE_NORMAL],
					 (gchar **) xpm_rtext);
  pixmapwid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmapwid);
  gdk_pixmap_unref (pixmap);

  gtk_container_add (GTK_CONTAINER (rtextbutton), pixmapwid);
  gtk_box_pack_start (GTK_BOX (hbox), rtextbutton, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (rtextbutton), "clicked",
		      GTK_SIGNAL_FUNC (Change_Text), (void *) -2);
  gtk_tooltips_set_tip (tooltips, rtextbutton, _("Draw right-justified text"),
			NULL);
  gtk_widget_show (rtextbutton);

  bondomenu = gtk_option_menu_new ();
  bondmenu = gtk_menu_new ();
  group = NULL;
  for (i = 0; i < bondtypes; i++)
    {
      int ii = i;

      menuitem = gtk_radio_menu_item_new (group);
      pixmap = gdk_pixmap_create_from_xpm_d (window->window, &mask,
					     &style->bg[GTK_STATE_NORMAL],
					     (gchar **) xpm_bond[i]);
      pixmapwid = gtk_pixmap_new (pixmap, mask);
      gtk_widget_show (pixmapwid);
      gdk_pixmap_unref (pixmap);
      gtk_container_add (GTK_CONTAINER (menuitem), pixmapwid);
      gtk_container_set_border_width (GTK_CONTAINER (menuitem), 0);

      group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (menuitem));
      gtk_menu_append (GTK_MENU (bondmenu), menuitem);
      gtk_widget_show (menuitem);
      gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
			  GTK_SIGNAL_FUNC (set_bond), (void *) ii);
    }
  gtk_option_menu_set_menu (GTK_OPTION_MENU (bondomenu), bondmenu);
  gtk_box_pack_start (GTK_BOX (hbox), bondomenu, FALSE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, bondomenu, _("Choose default bond type"),
			NULL);
  gtk_widget_show (bondomenu);

  bondbutton = gtk_toggle_button_new_with_label (_("Bonds"));
  gtk_box_pack_start (GTK_BOX (hbox), bondbutton, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (bondbutton), "clicked",
		      GTK_SIGNAL_FUNC (Bondmode), NULL);
  gtk_tooltips_set_tip (tooltips, bondbutton, _("Toggle bond types"), NULL);
  gtk_widget_show (bondbutton);


  markbutton = gtk_toggle_button_new ();
  pixmap = gdk_pixmap_create_from_xpm_d (window->window, &mask,
					 &style->bg[GTK_STATE_NORMAL],
					 (gchar **) xpm_mark);
  pixmapwid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmapwid);
  gdk_pixmap_unref (pixmap);
  gtk_container_add (GTK_CONTAINER (markbutton), pixmapwid);
  gtk_box_pack_start (GTK_BOX (hbox), markbutton, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (markbutton), "clicked",
		      GTK_SIGNAL_FUNC (Markmode), NULL);
  gtk_tooltips_set_tip (tooltips, markbutton,
			_("Mark objects for moving, rotating or deleting"),
			NULL);
  gtk_widget_show (markbutton);


  movebutton = gtk_toggle_button_new ();
  pixmap = gdk_pixmap_create_from_xpm_d (window->window, &mask,
					 &style->bg[GTK_STATE_NORMAL],
					 (gchar **) xpm_mov);
  pixmapwid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmapwid);
  gdk_pixmap_unref (pixmap);
  gtk_container_add (GTK_CONTAINER (movebutton), pixmapwid);
  gtk_box_pack_start (GTK_BOX (hbox), movebutton, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (movebutton), "clicked",
		      GTK_SIGNAL_FUNC (Movemode), NULL);
  gtk_tooltips_set_tip (tooltips, movebutton, _("Move marked object"), NULL);
  gtk_widget_show (movebutton);



  rotatebutton = gtk_toggle_button_new ();
  pixmap = gdk_pixmap_create_from_xpm_d (window->window, &mask,
					 &style->bg[GTK_STATE_NORMAL],
					 (gchar **) xpm_rot);
  pixmapwid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmapwid);
  gdk_pixmap_unref (pixmap);
  gtk_container_add (GTK_CONTAINER (rotatebutton), pixmapwid);
  gtk_box_pack_start (GTK_BOX (hbox), rotatebutton, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (rotatebutton), "clicked",
		      GTK_SIGNAL_FUNC (Rotatemode), NULL);
  gtk_tooltips_set_tip (tooltips, rotatebutton, _("Rotate marked object"),
			NULL);
  gtk_widget_show (rotatebutton);



  hflipbutton = gtk_button_new ();
  pixmap = gdk_pixmap_create_from_xpm_d (window->window, &mask,
					 &style->bg[GTK_STATE_NORMAL],
					 (gchar **) xpm_horiz);
  pixmapwid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmapwid);
  gdk_pixmap_unref (pixmap);
  gtk_container_add (GTK_CONTAINER (hflipbutton), pixmapwid);
  gtk_box_pack_start (GTK_BOX (hbox), hflipbutton, FALSE, TRUE, 0);
  gtk_signal_connect_object (GTK_OBJECT (hflipbutton), "clicked",
			     GTK_SIGNAL_FUNC (flip_horiz),
			     GTK_OBJECT (window));
  gtk_tooltips_set_tip (tooltips, hflipbutton, _("Flip object horizontally"),
			NULL);
  gtk_widget_show (hflipbutton);


  vflipbutton = gtk_button_new ();
  pixmap = gdk_pixmap_create_from_xpm_d (window->window, &mask,
					 &style->bg[GTK_STATE_NORMAL],
					 (gchar **) xpm_vert);
  pixmapwid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmapwid);
  gdk_pixmap_unref (pixmap);
  gtk_container_add (GTK_CONTAINER (vflipbutton), pixmapwid);
  gtk_box_pack_start (GTK_BOX (hbox), vflipbutton, FALSE, TRUE, 0);
  gtk_signal_connect_object (GTK_OBJECT (vflipbutton), "clicked",
			     GTK_SIGNAL_FUNC (flip_vert),
			     GTK_OBJECT (window));
  gtk_tooltips_set_tip (tooltips, vflipbutton, _("Flip object vertically"),
			NULL);
  gtk_widget_show (vflipbutton);



  copybutton = gtk_button_new ();
  pixmap = gdk_pixmap_create_from_xpm_d (window->window, &mask,
					 &style->bg[GTK_STATE_NORMAL],
					 (gchar **) xpm_copy);
  pixmapwid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmapwid);
  gdk_pixmap_unref (pixmap);
  gtk_container_add (GTK_CONTAINER (copybutton), pixmapwid);
  gtk_box_pack_start (GTK_BOX (hbox), copybutton, FALSE, TRUE, 0);
  gtk_signal_connect_object (GTK_OBJECT (copybutton), "clicked",
			     GTK_SIGNAL_FUNC (copy_obj), GTK_OBJECT (window));
  gtk_signal_connect_object (GTK_OBJECT (copybutton), "clicked",
			     GTK_SIGNAL_FUNC (gtk_toggle_button_set_active),GTK_OBJECT(movebutton));
  gtk_tooltips_set_tip (tooltips, copybutton, _("Copy marked object"), NULL);
  gtk_widget_show (copybutton);

  rescalebutton = gtk_toggle_button_new ();
  pixmap = gdk_pixmap_create_from_xpm_d (window->window, &mask,
					 &style->bg[GTK_STATE_NORMAL],
					 (gchar **) xpm_scale);
  pixmapwid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmapwid);
  gdk_pixmap_unref (pixmap);
  gtk_container_add (GTK_CONTAINER (rescalebutton), pixmapwid);
  gtk_box_pack_start (GTK_BOX (hbox), rescalebutton, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (rescalebutton), "clicked",
		      GTK_SIGNAL_FUNC (Rescalemode), NULL);
  gtk_tooltips_set_tip (tooltips, rescalebutton, _("Rescale marked object"),
			NULL);
  gtk_widget_show (rescalebutton);

#if 1
  bracketbutton = gtk_button_new ();
  pixmap = gdk_pixmap_create_from_xpm_d (window->window, &mask,
					 &style->bg[GTK_STATE_NORMAL],
					 (gchar **) xpm_bracket);
  pixmapwid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmapwid);
  gdk_pixmap_unref (pixmap);
  gtk_container_add (GTK_CONTAINER (bracketbutton), pixmapwid);
  gtk_box_pack_start (GTK_BOX (hbox), bracketbutton, FALSE, TRUE, 0);
  gtk_signal_connect_object (GTK_OBJECT (bracketbutton), "clicked",
			     GTK_SIGNAL_FUNC (add_bracket),
			     GTK_OBJECT (window));
  gtk_tooltips_set_tip (tooltips, bracketbutton, _("Draw brackets around object"),
			NULL);
  gtk_widget_show (bracketbutton);
#endif

  cleanbutton = gtk_button_new ();
  pixmap = gdk_pixmap_create_from_xpm_d (window->window, &mask,
					 &style->bg[GTK_STATE_NORMAL],
					 (gchar **) xpm_clean);
  pixmapwid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmapwid);
  gdk_pixmap_unref (pixmap);
  gtk_container_add (GTK_CONTAINER (cleanbutton), pixmapwid);
  gtk_box_pack_start (GTK_BOX (hbox), cleanbutton, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (cleanbutton), "clicked",
		      GTK_SIGNAL_FUNC (tidy_mol), NULL);
  gtk_tooltips_set_tip (tooltips, cleanbutton, _("Clean up drawing"), NULL);

  gtk_widget_show (cleanbutton);

  textbox = gtk_entry_new_with_max_length (80);
  gtk_box_pack_start (GTK_BOX (hbox), textbox, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (textbox), "focus_in_event",
                        (GtkSignalFunc) Change_Text, (void *) 0);
                        
  gtk_tooltips_set_tip (tooltips, textbox,
			_
			("Prefix a character with _ for subscript, ^ for superscript, @ for symbols, e.g. H_2O, Mo K_@a radiation, @a_D^2^0"),
			NULL);
  gtk_widget_show (textbox);

  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
  gtk_widget_show (hbox);

/*end second row */

/* drawing area */
/***********************the canvas pixmap ***********************/
  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_container_set_border_width (GTK_CONTAINER (scrolled_window), 10);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
				  GTK_POLICY_ALWAYS, GTK_POLICY_ALWAYS);
  gtk_box_pack_start (GTK_BOX (vbox), scrolled_window, TRUE, TRUE, 0);
  gtk_widget_show (scrolled_window);


  gtk_widget_set_usize (window, 800, 580);

  if (!picture)
    picture = gdk_pixmap_new (window->window, 800, 600, -1);

  drawing_area = gtk_drawing_area_new ();
  gtk_drawing_area_size (GTK_DRAWING_AREA (drawing_area), 1600, 1600);

  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW
					 (scrolled_window), drawing_area);

  gtk_widget_show (drawing_area);

  /* Signals used to handle backing pixmap */

  gtk_signal_connect (GTK_OBJECT (drawing_area), "expose_event",
		      (GtkSignalFunc) expose_event, NULL);
  gtk_signal_connect (GTK_OBJECT (drawing_area), "configure_event",
		      (GtkSignalFunc) configure_event, NULL);

  gtk_signal_connect (GTK_OBJECT (drawing_area), "motion_notify_event",
		      (GtkSignalFunc) motion_notify_event, NULL);

  gtk_signal_connect (GTK_OBJECT (drawing_area), "button_press_event",
		      (GtkSignalFunc) button_press_event, NULL);

  gtk_signal_connect (GTK_OBJECT (drawing_area), "button_release_event",
		      (GtkSignalFunc) button_release_event, NULL);

  gtk_signal_connect (GTK_OBJECT (window), "key_press_event",
		      (GtkSignalFunc) key_press_event, NULL);

  gtk_widget_set_events (drawing_area, GDK_EXPOSURE_MASK
			 | GDK_LEAVE_NOTIFY_MASK
			 | GDK_BUTTON_PRESS_MASK
			 | GDK_BUTTON_RELEASE_MASK
			 | GDK_POINTER_MOTION_MASK
			 | GDK_POINTER_MOTION_HINT_MASK);

/*status line - used for cht output*/
  hbox = gtk_hbox_new (FALSE, 5);
  gtk_widget_set_usize (hbox, 800, 20);
  msgtext = gtk_text_new (NULL, NULL);
  gtk_text_set_editable (GTK_TEXT (msgtext), FALSE);
  gtk_box_pack_start (GTK_BOX (hbox), msgtext, TRUE, TRUE, 0);
  vscroll = gtk_vscrollbar_new (GTK_TEXT (msgtext)->vadj);
  gtk_box_pack_start (GTK_BOX (hbox), vscroll, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
  gtk_text_insert (GTK_TEXT (msgtext), NULL, NULL, NULL, _("Ready     "), 10);
  gtk_widget_show (msgtext);
  gtk_widget_show (vscroll);
  gtk_widget_show (hbox);
  gtk_widget_show (window);

/* initialize cursor shapes */
  source =
    gdk_bitmap_create_from_data (NULL, pencil_bits, pencil_width,
				 pencil_height);
  mask =
    gdk_bitmap_create_from_data (NULL, pencil_mask_bits, pencil_width,
				 pencil_height);
  cursor_pencil =
    gdk_cursor_new_from_pixmap (source, mask, &fg, &bg, pencil_x_hot,
				pencil_y_hot);
  gdk_pixmap_unref (source);
  gdk_pixmap_unref (mask);
  source =
    gdk_bitmap_create_from_data (NULL, text_bits, text_width, text_height);
  mask =
    gdk_bitmap_create_from_data (NULL, text_mask_bits, text_width,
				 text_height);
  cursor_text =
    gdk_cursor_new_from_pixmap (source, mask, &fg, &bg, text_x_hot,
				text_y_hot);
  gdk_pixmap_unref (source);
  gdk_pixmap_unref (mask);
  source =
    gdk_bitmap_create_from_data (NULL, bonds_bits, bonds_width, bonds_height);
  mask =
    gdk_bitmap_create_from_data (NULL, bonds_mask_bits, bonds_width,
				 bonds_height);
  cursor_bonds =
    gdk_cursor_new_from_pixmap (source, mask, &fg, &wh, bonds_x_hot,
				bonds_y_hot);
  gdk_pixmap_unref (source);
  gdk_pixmap_unref (mask);
  cursor_markTLC = gdk_cursor_new (GDK_TOP_LEFT_CORNER);
  cursor_markBRC = gdk_cursor_new (GDK_BOTTOM_RIGHT_CORNER);
  cursor_move = gdk_cursor_new (GDK_FLEUR);
  cursor_rescale = gdk_cursor_new (GDK_SIZING);
  /* cursor_bonds = gdk_cursor_new (GDK_DRAPED_BOX); */
  cursor_rotate = gdk_cursor_new (GDK_EXCHANGE);
  cursor_busy = gdk_cursor_new (GDK_WATCH);



  gtk_toggle_button_set_active ((GtkToggleButton *) hexbutton, TRUE);
  /* default to linedrawing at hexagonal angles */
  text_direct = -1;		/* and centered text */
  expmode = 0;			/*default to xfig export */
  strcpy (filename, _("unnamed"));
    gtk_file_selection_set_filename (GTK_FILE_SELECTION (expw), filename);
  mark.flag = False;
  head.width = 2000;
  head.height = 5000;
  zoom_factor = 2;
  hp.x = hp.y = -1;
  if (argc == 2)
    {
      strcpy (filename, argv[1]);
      load_mol (filename);
    }
  FreePix ();
  CreatePix ();
  Display_Mol ();

  gtk_main ();			/* enter the gtk eventloop */

  return 0;
}
